% Script-File 

% Hinzuf"ugen eines Suchpfades f"ur M-Files (nur falls erforderlich)
addpath '../routines/';
addpath '../examples/';


% Vektor mit Dimension der Matrix
v=[10 50 100 200 400];

% Mehrfach Cholesky-Zerlegung testen.
Anz=3;

k=menu('Auswahl der Testbeispiele', ...
       'MATLAB', ...
       'chol', ...
       'Cholesky-Zerl. der Vorl.', ...
       'Vergleich aller drei',...
       'Ende');

while (k<5)

       % Vektoren f"ur Zeit, Fehler, Konditionszahl
       zeit  =zeros(3,length(v));
       fehler=zeros(3,length(v));
       i=1;
       for n=v
	   for j=1:Anz
	       % Beliebige Matrix der Dimension n
	       A=rand(n);
	       A=A*diag((1:n).*(1:n))*A';
	       % Beliebige rechte Seite
	       x=rand(n,1); b=A*x;

	       % 1. Variante, eingebaute LU-Variante mittels backslash
	       if (k==1 | k==4)
		  t0=clock;
		  y=A\b;
		  zeit(1,i)=zeit(1,i)+etime(clock,t0);
		  fehler(1,i)=fehler(1,i)+norm(x-y)/norm(x);
	       end; % if

	       % 2. Variante, chol aus MATLAB 
	       if (k==2 | k==4)
		  % 2. Variante, chol(A)
		  R=A; y=b;
		  t0=clock;
		  % A=R'*R
		  R=chol(R);
		  y=R\(R'\y);
		  zeit(2,i)=zeit(2,i)+etime(clock,t0);
		  fehler(2,i)=fehler(2,i)+norm(x-y)/norm(x);       
	       end % if

	       % 3. Variante, Cholesky-Zerlegung der Vorlesung
	       if (k==3 | k==4)
		  G=tril(A); y=b;
		  t0=clock;
		  G=CHOLESKY(G); 
		  % Vorw"artseinsetzen 
		  y=CHOLFORWARD(G,y);
		  % R"uckw"artseinsetzen
		  y=CHOLBACKWARD(G,y);
		  zeit(3,i)=zeit(3,i)+etime(clock,t0);
		  fehler(3,i)=fehler(3,i)+norm(x-y)/norm(x);	       
	       end % if
	   end; % for
           zeit(:,i)  =zeit(:,i)/Anz;
	   fehler(:,i)=fehler(:,i)/Anz;
	   i=i+1;
       end % for

       figure(1);
       hold off;
       if (k==1 | k==4)
	  semilogy(v,fehler(1,:),'+r');
	  hold on;
       end; % if
       if (k==2 | k==4)
 	  semilogy(v,fehler(2,:),'og');
       end; % if
       if (k==3 | k==4)
 	  semilogy(v,fehler(3,:),'*b');
       end % if
       
       hold off;
       vv=axis;
       axis([vv(1)-1 vv(2)+10 vv(3:4)]);
       title('Relative Fehler der Loesung');
       xlabel('n');
       ylabel('||e||/||x||');

       figure(2);
       hold off;
       if (k==1 | k==4)
	  semilogy(v,zeit(1,:),'+r');
	  hold on;
       end; % if
       if (k==2 | k==4)
 	  semilogy(v,zeit(2,:),'og');
       end; % if
       if (k==3 | k==4)
 	  semilogy(v,zeit(3,:),'*b');
       end; % if
       hold off;
       hold off;
       vv=axis;
       axis([vv(1)-1 vv(2)+10 vv(3:4)]);
       title('Rechenzeit');
       xlabel('n');
       ylabel('[sec]');

       for ii=1:2
	  figure(ii);
	  if k==1
	     legend('MATLAB(\\)',4);
	  end % if
	  if k==2 
	     legend('chol(MATLAB)',4);
	  end % if
	  if k==3 
	     legend('CHOLESKY',4);
	  end % if
	  if k==4
	     legend('MATLAB(\\)',...
		    'chol(MATLAB)',...
		    'CHOLESKY',4);
	  end % if
       end % for ii
       
       k=menu('Auswahl der Testbeispiele', ...
	      'MATLAB', ...
	      'chol', ...
	      'Cholesky-Zerl. der Vorl.', ...
	      'Vergleich aller drei',...
	      'Ende');
end; % while




